from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class AutoRoleMigration(MigrationABC):
    name = '0.2.1_AutoRoleMigration'

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, 'Running upgrade')

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `AutoRoles` (
                `AutoRoleId` BIGINT NOT NULL AUTO_INCREMENT,
                `ServerId` BIGINT,
                `DiscordMessageId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`AutoRoleId`),
                FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `AutoRoleRules` (
                `AutoRoleRuleId` BIGINT NOT NULL AUTO_INCREMENT,
                `AutoRoleId` BIGINT,
                `DiscordEmojiName` VARCHAR(64),
                `DiscordRoleId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`AutoRoleRuleId`),
                FOREIGN KEY (`AutoRoleId`) REFERENCES `AutoRoles`(`AutoRoleId`)
            );
            """)
        )

    def downgrade(self):
        self._cursor.execute('DROP TABLE `AutoRole`;')
        self._cursor.execute('DROP TABLE `AutoRoleRules`;')
