import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.container import Guild, Role, Member
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.level import Level
from bot_data.model.user import User
from bot_data.service.level_repository_service import LevelRepositoryService
from bot_data.service.server_repository_service import ServerRepositoryService
from bot_data.service.user_repository_service import UserRepositoryService


class LevelService:

    def __init__(
            self,
            logger: LoggerABC,
            db: DatabaseContextABC,
            levels: LevelRepositoryService,
            users: UserRepositoryService,
            servers: ServerRepositoryService,
            bot: DiscordBotServiceABC
    ):
        self._logger = logger
        self._db = db
        self._levels = levels
        self._users = users
        self._servers = servers
        self._bot = bot

    def get_level(self, user: User) -> Level:
        levels = self._levels.get_levels_by_server_id(user.server.server_id).order_by(lambda l: l.min_xp)
        return levels.where(lambda l: user.xp >= l.min_xp).first()

    async def set_level(self, user: User):
        level_names = self._levels.get_levels_by_server_id(user.server.server_id).select(lambda l: l.name)
        guild: Guild = self._bot.guilds.where(lambda g: g.id == user.server.discord_server_id).single()
        level_role: Role = guild.roles.where(lambda r: r.name == self.get_level(user).name).single()
        member: Member = guild.members.where(lambda m: m.id == user.discord_id).single()
        for role in member.roles:
            if role.name not in level_names.to_list():
                continue

            try:
                self._logger.debug(__name__, f'Try to remove role {role.name} from {member.name}')
                await member.remove_roles(role)
                self._logger.info(__name__, f'Removed role {role.name} from {member.name}')
            except Exception as e:
                self._logger.error(__name__, f'Removing role {role.name} from {member.name} failed!', e)

        if level_role in member.roles:
            return
        try:
            self._logger.debug(__name__, f'Try to add role {level_role.name} to {member.name}')
            await member.add_roles(level_role)
            self._logger.info(__name__, f'Add role {level_role.name} to {member.name}')
        except Exception as e:
            self._logger.error(__name__, f'Adding role {level_role.name} to {member.name} failed!', e)

    async def check_level(self, member: discord.Member):
        if member.bot:
            return

        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.find_user_by_discord_id_and_server_id(member.id, server.server_id)
        if user is None:
            self._logger.warn(__name__, f'User not found {member.guild.name}@{member.name}')

        await self.set_level(user)
